@@ -18,29 +18,30 @@ class PublishManager(object):
     """
     Manage multiple publishes
     """
-    def __init__(self, client):
+    def __init__(self, client, storage=""):
         self.client = client
         self._publishes = {}
+        self.storage = storage
         self.timestamp = int(time.time())
 
-    def publish(self, distribution):
+    def publish(self, distribution, storage=""):
         """
         Get or create publish
         """
         try:
             return self._publishes[distribution]
         except KeyError:
-            self._publishes[distribution] = Publish(self.client, distribution, timestamp=self.timestamp)
+            self._publishes[distribution] = Publish(self.client, distribution, timestamp=self.timestamp, storage=(storage or self.storage))
             return self._publishes[distribution]
 
-    def add(self, snapshot, distributions, component='main'):
+    def add(self, snapshot, distributions, component='main', storage=""):
         """ Add mirror or repo to publish """
         for dist in distributions:
-            self.publish(dist).add(snapshot, component)
+            self.publish(dist, storage=storage).add(snapshot, component)
 
     def restore_publish(self, components, restore_file, recreate):
         publish_file = load_publish(restore_file)
-        publish_source = Publish(self.client, publish_file.get('publish'))
+        publish_source = Publish(self.client, publish_file.get('publish'), storage=publish_file.get('storage', self.storage))
         publish_source.restore_publish(publish_file,
                                        components=components,
                                        recreate=recreate)
@@ -58,15 +59,15 @@ class PublishManager(object):
 
         publishes = self.client.do_get('/publish')
         for publish in publishes:
-            name = '{}/{}'.format(publish['Prefix'], publish['Distribution'])
+            name = '{}/{}'.format(publish['Prefix'].replace("/", "_"), publish['Distribution'])
             if save_all or name in publishes_to_save:
-                save_list.append(Publish(self.client, name, load=True))
+                save_list.append(Publish(self.client, name, load=True, storage=publish.get('Storage', self.storage)))
 
         if not save_all and len(save_list) != len(publishes_to_save):
             raise Exception('Publish(es) required not found')
 
         for publish in save_list:
-            save_path = ''.join([dump_dir, '/', prefix, publish.name.replace('/', '-')])
+            save_path = ''.join([dump_dir, '/', prefix, publish.name.replace('/', '-'), '.yml'])
             publish.save_publish(save_path)
 
     def _publish_match(self, publish, names=False, name_only=False):
@@ -109,6 +110,45 @@ class PublishManager(object):
             keys[e] = 1
         return list(keys.keys())
 
+    def do_purge(self, config, components=[], hard_purge=False):
+        (repo_dict, publish_dict) = self.get_repo_information(config, self.client, hard_purge, components)
+        publishes = self.client.do_get('/publish')
+        publish_list = []
+
+        for publish in publishes:
+            name = '{}/{}'.format(publish['Prefix'].replace("/", "_"), publish['Distribution'])
+            publish_list.append(Publish(self.client, name, load=True))
+
+        for publish in publish_list:
+            repo_dict = publish.purge_publish(repo_dict, publish_dict, components, publish=True)
+
+        if hard_purge:
+            self.remove_unused_packages(repo_dict)
+            self.cleanup_snapshots()
+
+    @staticmethod
+    def get_repo_information(config, client, fill_repo=False, components=[]):
+        """ fill two dictionnaries : one containing all the packages for every repository
+            and the second one associating to every component of every publish its repository"""
+        repo_dict = {}
+        publish_dict = {}
+
+        for name, repo in config.get('repo', {}).items():
+            if components and repo.get('component') not in components:
+                continue
+            if fill_repo:
+                packages = client.do_get('/{}/{}/{}'.format("repos", name, "packages"))
+                repo_dict[name] = packages
+            for distribution in repo.get('distributions'):
+                publish_dict[(distribution.split('/')[0], repo.get('component'))] = name
+
+        return (repo_dict, publish_dict)
+
+    def remove_unused_packages(self, repo_dict):
+        for repo_name, packages in repo_dict.items():
+            if packages:
+                self.client.do_delete('/repos/%s/packages' % repo_name, data={'PackageRefs': packages})
+
     def cleanup_snapshots(self):
         snapshots = self.client.do_get('/snapshots', {'sort': 'time'})
         exclude = []
@@ -149,14 +189,15 @@ class Publish(object):
     """
     Single publish object
     """
-    def __init__(self, client, distribution, timestamp=None, recreate=False, load=False, merge_prefix='_'):
+    def __init__(self, client, distribution, timestamp=None, recreate=False, load=False, merge_prefix='_', storage=""):
         self.client = client
         self.recreate = recreate
+        self.storage = storage
 
         dist_split = distribution.split('/')
         self.distribution = dist_split[-1]
         if dist_split[0] != self.distribution:
-            self.prefix = dist_split[0]
+            self.prefix = "_".join(dist_split[:-1])
         else:
             self.prefix = ''
 
@@ -222,7 +263,8 @@ class Publish(object):
         publishes = self.client.do_get('/publish')
         for publish in publishes:
             if publish['Distribution'] == self.distribution and \
-                    publish['Prefix'] == (self.prefix or '.'):
+                    publish['Prefix'].replace("/", "_") == (self.prefix or '.') and \
+                    publish['Storage'] == self.storage:
                 return publish
         raise NoSuchPublish("Publish %s does not exist" % self.name)
 
@@ -241,6 +283,7 @@ class Publish(object):
         yaml_dict["publish"] = self.name
         yaml_dict["name"] = timestamp
         yaml_dict["components"] = []
+        yaml_dict["storage"] = self.storage
         for component, snapshots in self.components.items():
             packages = self.get_packages(component)
             package_dict = []
@@ -256,6 +299,76 @@ class Publish(object):
         with open(save_path, 'w') as save_file:
             yaml.dump(yaml_dict, save_file, default_flow_style=False)
 
+    def purge_publish(self, repo_dict, publish_dict, components=[], publish=False):
+
+        new_publish_snapshots = []
+
+        for snapshot in self.publish_snapshots:
+            # packages to be kept
+            processed = []
+            name = snapshot["Name"]
+            component = snapshot["Component"]
+            purge_packages = []
+            location = self.name.split('/')[0]
+            try:
+                repo_name = publish_dict[(location, component)]
+            except KeyError:
+                new_publish_snapshots.append(snapshot)
+                continue
+
+            if components and component not in components:
+                new_publish_snapshots.append(snapshot)
+                if repo_dict:
+                    repo_dict[repo_name] = []
+                continue
+
+            packages = self.client.do_get('/{}/{}/{}'.format("snapshots", name, "packages"))
+            packages = sorted(packages, key=lambda x: self.parse_package_ref(x)[2], reverse=True)
+
+            for package in packages:
+                package_name = self.parse_package_ref(package)[1]
+
+                if package_name not in processed:
+                    processed.append(package_name)
+                    if repo_dict and package in repo_dict[repo_name]:
+                        repo_dict[repo_name].remove(package)
+
+                    purge_packages.append(package)
+
+            if purge_packages != packages:
+                snapshot_name = '{}-{}'.format(name, 'purged')
+                try:
+                    lg.debug("Creating new snapshot: %s" % snapshot_name)
+                    self.client.do_post(
+                        '/snapshots',
+                        data={
+                            'Name': snapshot_name,
+                            'SourceSnapshots': [],
+                            'Description': 'Minimal snapshot from repo {}'.format(repo_name),
+                            'PackageRefs': purge_packages,
+                        }
+                    )
+
+                except AptlyException as e:
+                    if e.res.status_code == 404:
+                        raise Exception('Error while creating snapshot : {}'.format(repr(e)))
+                    else:
+                        lg.debug("Snapshot %s already exist" % snapshot_name)
+
+                new_publish_snapshots.append({
+                    'Component': component,
+                    'Name': snapshot_name
+                })
+            else:
+                new_publish_snapshots.append(snapshot)
+
+        if self.publish_snapshots != new_publish_snapshots:
+            self.publish_snapshots = new_publish_snapshots
+            if publish:
+                self.do_publish(recreate=False, merge_snapshots=False)
+
+        return repo_dict
+
     def restore_publish(self, config, components, recreate=False):
         """
         Restore publish from config file
@@ -472,16 +585,18 @@ class Publish(object):
             })
 
     def drop_publish(self):
-        lg.info("Deleting publish, distribution=%s" % self.name)
+        lg.info("Deleting publish, distribution=%s, storage=%s" % (self.name, self.storage or "local"))
 
         self.client.do_delete('/publish/%s/%s' % (self.prefix, self.distribution))
 
     def update_publish(self, force_overwrite=False, publish_contents=False):
-        lg.info("Updating publish, distribution=%s snapshots=%s" %
-                (self.name, self.publish_snapshots))
+        lg.info("Updating publish, distribution=%s storage=%s snapshots=%s" %
+                (self.name, self.storage or "local", self.publish_snapshots))
+
+        prefix = '%s%s' % (self.storage+":" or "", self.prefix)
 
         self.client.do_put(
-            '/publish/%s/%s' % (self.prefix, self.distribution),
+            '/publish/%s/%s' % (prefix, self.distribution),
             {
                 'Snapshots': self.publish_snapshots,
                 'ForceOverwrite': force_overwrite,
@@ -490,13 +605,16 @@ class Publish(object):
         )
 
     def create_publish(self, force_overwrite=False, publish_contents=False, architectures=None):
-        lg.info("Creating new publish, distribution=%s snapshots=%s, architectures=%s" %
-                (self.name, self.publish_snapshots, architectures))
+        lg.info("Creating new publish, distribution=%s storage=%s snapshots=%s, architectures=%s" %
+                (self.name, self.storage or "local", self.publish_snapshots, architectures))
 
         if self.prefix:
-            prefix = '/%s' % self.prefix
+            prefix = '%s%s' % ("/"+self.storage+":" or "/", self.prefix)
+        else:
+            prefix = '%s' % ("/"+self.storage+":" or "")
 
         opts = {
+            "Storage": self.storage,
             "SourceKind": "snapshot",
             "Distribution": self.distribution,
             "Sources": self.publish_snapshots,
@@ -514,7 +632,8 @@ class Publish(object):
 
     def do_publish(self, recreate=False, no_recreate=False,
                    force_overwrite=False, publish_contents=False,
-                   architectures=None, merge_snapshots=True):
+                   architectures=None, merge_snapshots=True,
+                   only_latest=False, config=None, components=[]):
         if merge_snapshots:
             self.merge_snapshots()
         try:
@@ -522,9 +641,14 @@ class Publish(object):
         except NoSuchPublish:
             publish = False
 
+        if only_latest:
+            (_, publish_dict) = PublishManager.get_repo_information(config, self.client)
+            self.purge_publish([], publish_dict, components, False)
+
         if not publish:
             # New publish
-            self.create_publish(force_overwrite, publish_contents, architectures)
+            self.create_publish(force_overwrite, publish_contents,
+                                architectures)
         else:
             # Test if publish is up to date
             to_publish = [x['Name'] for x in self.publish_snapshots]
@@ -536,7 +660,8 @@ class Publish(object):
             if recreate:
                 lg.info("Recreating publish %s" % self.name)
                 self.drop_publish()
-                self.create_publish(force_overwrite, publish_contents, architectures)
+                self.create_publish(force_overwrite, publish_contents,
+                                    architectures)
             elif to_publish == published:
                 lg.info("Publish %s is up to date" % self.name)
             else:
@@ -551,4 +676,6 @@ class Publish(object):
                         else:
                             lg.warning("Cannot update publish %s (adding new components?), falling back to recreating it" % self.name)
                             self.drop_publish()
-                            self.create_publish(force_overwrite, publish_contents, architectures)
+                            self.create_publish(force_overwrite,
+                                                publish_contents,
+                                                architectures)
